//**************************************************************************
//*                     This file is part of the                           *
//*                MMC - Mpxplay Multimedia Commander                      *
//*                   The source code of MMC is                            *
//*        (C) copyright 1998-2019 by PDSoft (Attila Padar)                *
//*                http://mpxplay.sourceforge.net                          *
//*                  email: mpxplay@freemail.hu                            *
//**************************************************************************
//*  This program is distributed in the hope that it will be useful,       *
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
//*  Please contact with the author (with me) if you want to use           *
//*  or modify this source.                                                *
//**************************************************************************
//function: video software based transformations (crop, pad, eq)

//#define MPXPLAY_USE_DEBUGF 1
#define DISPQT_DEBUG_OUTPUT NULL //stdout
#define DISPQT_DEBUG_ERROR stderr
#define DISPQT_DEBUGOUT_FFMPEG stdout // NULL
#define DISPQT_DEBUGOUT_RENDER NULL // stdout
#define DISPQT_DEBUGOUT_CROP NULL // stdout
#define DISPQT_DEBUGOUT_ACROP NULL // stdout

#include "moc_video_qt.h"
#include "moc_mainwindow.h"

#ifdef MPXPLAY_LINK_ORIGINAL_FFMPEG

FFMpegVideoSoftwareTransform::FFMpegVideoSoftwareTransform(MainWindow *main_window, dispqt_video_source_info_s *video_source_infos, dispqt_video_surface_info_s *video_surface_infos)
{
	struct mmc_dispqt_config_s *gcfg = main_window->gui_config;
	this->main_window = main_window;
	this->video_source_infos = video_source_infos;
	this->video_surface_infos = video_surface_infos;

	this->videowidget_ffmpeg_swsctx_crop_auto_clear();
	this->video_source_infos->last_video_crop_type = gcfg->video_crop_type;

	memset(&this->video_filter_infos, 0, sizeof(this->video_filter_infos));

	this->videowidget_ffmpeg_videoctx_init(&this->video_filter_infos);
	this->videowidget_ffmpeg_swsctx_clear(&this->video_filter_infos);
}

FFMpegVideoSoftwareTransform::~FFMpegVideoSoftwareTransform()
{
	videowidget_ffmpeg_swsctx_close();
	videowidget_ffmpeg_videoctx_close(&this->video_filter_infos);
}

// --------------------------------------------------------------------------------------------------------------------

int FFMpegVideoSoftwareTransform::videowidget_ffmpeg_videoctx_init(struct dispqt_video_filter_info_s *video_filter_infos)
{
	int retcode = MPXPLAY_ERROR_OK;
	this->videowidget_ffmpeg_videoctx_clear();
	video_filter_infos->ffmpeg_filtered_frame = av_frame_alloc();
	video_filter_infos->ffmpeg_filtered_frame->format = this->video_surface_infos->videoout_surface_pix_fmt;
	return retcode;
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_videoctx_close(struct dispqt_video_filter_info_s *video_filter_infos)
{
	this->videowidget_ffmpeg_videoctx_clear();
	av_frame_free(&video_filter_infos->ffmpeg_transformed_frame);
	av_frame_free(&video_filter_infos->ffmpeg_filtered_frame);
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_videoctx_clear(void)
{
	struct dispqt_video_filter_info_s *video_filter_infos = &this->video_filter_infos;
	av_frame_free(&this->video_source_infos->ffmpeg_decoded_frame);
	this->video_surface_infos->ffmpeg_output_frame = NULL;
}

//-------------------------------------------------------------------------------------------------------------------------------------

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_filtered_crop_update(bool check_autocrop)
{
	struct dispqt_video_source_info_s *video_source_infos = this->video_source_infos;
	struct dispqt_video_filter_info_s *video_filter_infos = &this->video_filter_infos;
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	if(video_source_infos->last_video_crop_type != gcfg->video_crop_type)
	{
		video_source_infos->last_video_crop_type = gcfg->video_crop_type;
		if(!video_source_infos->ffmpeg_swsframe_cropreset)
			video_source_infos->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_REFRESH;
		video_source_infos->sws_autoxcrop_framecounter = 0;
		check_autocrop = true;
	}
	if((gcfg->video_crop_type == DISPQT_VIDEOSCRCROPTYPE_AUTO) && check_autocrop)
	{
		AVFrame *autocrop_input_frame = ((video_filter_infos->ffmpeg_swsframe_hasdata)? video_filter_infos->ffmpeg_sws_frame : ((video_filter_infos->ffmpeg_transformed_frame)? video_filter_infos->ffmpeg_transformed_frame : this->video_source_infos->ffmpeg_decoded_frame)); // if the video is RGB, we use the original frame
		if(this->videowidget_ffmpeg_swsctx_crop_auto_detect(autocrop_input_frame, video_source_infos, &this->video_surface_infos->videoout_surface_clearpic))
		{
			this->video_surface_infos->videoout_surface_recalc = true;
		}
	}
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_filtered_trans_update(void)
{
	struct dispqt_video_filter_info_s *video_filter_infos = &this->video_filter_infos;

	av_frame_free(&video_filter_infos->ffmpeg_transformed_frame);
	if(this->video_source_infos->ffmpeg_decoded_frame)
	{
		DispQtVideoFrameFilter *vfilter_process_trans = this->main_window->qt_video_player->video_frame_filter_transform_process;
		if(vfilter_process_trans->video_filter_check(this->video_source_infos->ffmpeg_decoded_frame, AV_PIX_FMT_NONE) != AV_PIX_FMT_NONE)
		{
			DispQtVideoFrameFilter *vfilter_process_eq = this->main_window->qt_video_player->video_frame_filter_eq_process;
			int frameout_format = (vfilter_process_eq->video_filter_check(this->video_source_infos->ffmpeg_decoded_frame, AV_PIX_FMT_NONE) == AV_PIX_FMT_NONE)? this->video_surface_infos->videoout_surface_pix_fmt : DISPQT_VIDEO_FILTERPROCESS_AVFORMAT;
			vfilter_process_trans->video_filter_apply(this->video_source_infos->ffmpeg_decoded_frame, &video_filter_infos->ffmpeg_transformed_frame, frameout_format);
		}
		video_filter_infos->ffmpeg_swsframe_hasdata = videowidget_ffmpeg_swsctx_conv_frame(video_filter_infos);
		this->videowidget_ffmpeg_filtered_crop_update(true);
	}
	this->videowidget_ffmpeg_filtered_eq_update();
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_filtered_eq_update(void)
{
	struct dispqt_video_filter_info_s *video_filter_infos = &this->video_filter_infos;
	AVFrame *filter_in_frame;

	this->video_surface_infos->ffmpeg_output_frame = NULL;
	av_frame_unref(video_filter_infos->ffmpeg_filtered_frame);
	filter_in_frame = (video_filter_infos->ffmpeg_transformed_frame)? video_filter_infos->ffmpeg_transformed_frame : this->video_source_infos->ffmpeg_decoded_frame;
	if(filter_in_frame)
	{
		DispQtVideoFrameFilter *vfilter_process_eq = this->main_window->qt_video_player->video_frame_filter_eq_process;
		bool swsframe_for_outsurface = videowidget_ffmpeg_swsctx_check_swsframe_for_outsurface(video_filter_infos);
		if((!swsframe_for_outsurface && (filter_in_frame->format != this->video_surface_infos->videoout_surface_pix_fmt)) || (vfilter_process_eq->video_filter_check(filter_in_frame, AV_PIX_FMT_NONE) != AV_PIX_FMT_NONE))
		{
			int retVal = vfilter_process_eq->video_filter_apply(filter_in_frame, &video_filter_infos->ffmpeg_filtered_frame, this->video_surface_infos->videoout_surface_pix_fmt);
			if(retVal & DISPQT_VIDEOFILTER_APPLYRETBIT_OUTPUTFRAME_PRODUCED)
				this->video_surface_infos->ffmpeg_output_frame = video_filter_infos->ffmpeg_filtered_frame;
			mpxplay_debugf(DISPQT_DEBUGOUT_RENDER, "eq_update r:%d sw:%d sh:%d dw:%d dh:%d", retVal, filter_in_frame->width, filter_in_frame->height, video_filter_infos->ffmpeg_output_frame->width, video_filter_infos->ffmpeg_output_frame->height);
		}
		else
		{
			this->video_surface_infos->ffmpeg_output_frame = (swsframe_for_outsurface)? video_filter_infos->ffmpeg_sws_frame : filter_in_frame;
		}
	}
}

//-------------------------------------------------------------------------------------------------------------------------------------

#if DISPQT_VIDEO_AUTOCROP_USE_RGB32
#define DISPQT_VIDEO_AUTOCROP_RGB32_LIMITMASK 0x00e0e0e0
#else
#define DISPQT_VIDEO_AUTOCROP_YUV8_LIMITMASK 0xc0  // 8 bit planar Y values
#endif

bool FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_crop_auto_detect(AVFrame *input_video_frame, dispqt_video_source_info_s *video_source_infos, mpxp_bool_t *videooutsurface_clearpic)
{
	if(!input_video_frame)
		return false;
#if DISPQT_VIDEO_AUTOCROP_USE_RGB32
	const uint32_t limit_mask = DISPQT_VIDEO_AUTOCROP_RGB32_LIMITMASK;
#else
	const uint32_t limit_mask = DISPQT_VIDEO_AUTOCROP_YUV8_LIMITMASK;
#endif
	const int skip_border = DISPQT_CONFIG_VIDEO_AUTOCROP_SKIP_BORDER;
	const int check_range = 100 / DISPQT_CONFIG_VIDEO_AUTOCROP_CHECK_RANGE;
	uint8_t *video_data = input_video_frame->data[0];
	const int width = input_video_frame->width, height = input_video_frame->height;
	int x, y, check_len, counter, crop_x = 0, crop_y = 0, crop_width = 0, crop_height = 0;
#if DISPQT_VIDEO_AUTOCROP_USE_RGB32
	uint32_t *data32_begin = (uint32_t *)video_data, *data32;
#else
	uint8_t *data32_begin = (uint8_t *)video_data, *data32;
#endif
	bool retval = false;

	if(video_source_infos->ffmpeg_swsframe_cropreset == DISPQT_VIDEO_AUTOCROP_COUNTER_INIT){
		video_source_infos->sws_srccrop_x = 0;
		video_source_infos->sws_srccrop_y = 0;
		video_source_infos->sws_srccrop_width = width;
		video_source_infos->sws_srccrop_height = height;
		video_source_infos->ffmpeg_swsframe_cropreset --;
		video_source_infos->sws_autoxcrop_framecounter = 0;
		*videooutsurface_clearpic = true;
		retval = true;
		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP0 x:%d y:%d w:%d h:%d sw:%d sh:%d", crop_x, crop_y, crop_width, crop_height, input_video_frame->width, input_video_frame->height);
	}
	if(!data32_begin || (width < 32) || (height < 32) || (input_video_frame->format != DISPQT_VIDEO_AUTOCROP_AVFORMAT))
		return false;
	if(video_source_infos->sws_autoxcrop_framecounter > 0){
		video_source_infos->sws_autoxcrop_framecounter--;
		return false;
	}

	// check top side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + x;
	check_len = height / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto1 data:%8.8X m:%8.8X x:%d y:%d", *data32, limit_mask, x,y);
			break;
		}
		data32++;
		if(++x >= (width - skip_border)){
			x = skip_border; y++;
			data32 = data32_begin + y * width + x;
		}
	}while(y < check_len);

	if(y >= check_len)  // probably full (or partially) black picture
		goto err_out_autocrop_black;
	if(y > skip_border)
		crop_y = y;

	// check bottom side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + (height - y) * width + x;
	check_len = height / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto1 data:%8.8X m:%8.8X c:%d", *data32, limit_mask, counter);
			break;
		}
		data32++;
		if(++x >= (width - skip_border)){
			x = skip_border; y++;
			data32 = data32_begin + (height - y) * width + x;
		}
	}while(y < check_len);

	if(y >= check_len)  // partially black picture
		goto err_out_autocrop_black;

	if(video_source_infos->ffmpeg_swsframe_cropreset){ // top and bottom side crop shall be the same initially
		if(y > crop_y)
			y = crop_y;
		else if(crop_y > y)
			crop_y = y;
	}

	crop_height = height - crop_y;
	if((y > skip_border) && (y < check_len))
		crop_height -= y;

	//check left side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + x;
	check_len = width / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto3 data:%8.8X m:%8.8X x:%d", *data32, limit_mask, x);
			break;
		}
		data32 += width;
		if(++y >= (height - skip_border)){
			x++; y = skip_border;
			data32 = data32_begin + y * width + x;
		}
	}while(x < check_len);

	if(x >= check_len) // partially black picture
		goto err_out_autocrop_black;
	if((x > skip_border) && (x < check_len))
		crop_x = x - 1;

	//check right side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + width - x;
	check_len = width / check_range;
	do{
		if(*data32 & limit_mask)
			break;
		data32 += width;
		if(++y >= (height - skip_border)){
			x++; y = skip_border;
			data32 = data32_begin + y * width + width - x;
		}
	}while(x < check_len);

	if(x >= check_len) // partially black picture
		goto err_out_autocrop_black;

	if(video_source_infos->ffmpeg_swsframe_cropreset){ // left and right side crop shall be the same initially
		if(x > crop_x)
			x = crop_x;
		else if(crop_x > x)
			crop_x = x;
	}

	crop_width = width - crop_x;
	if((x > skip_border) && (x < check_len))
		crop_width -= x - 1;

	if(video_source_infos->ffmpeg_swsframe_cropreset){
		video_source_infos->sws_srccrop_x = crop_x;
		video_source_infos->sws_srccrop_y = crop_y;
		video_source_infos->sws_srccrop_width = crop_width;
		video_source_infos->sws_srccrop_height = crop_height;
		video_source_infos->ffmpeg_swsframe_cropreset --;
		*videooutsurface_clearpic = true;
		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP1 x:%d y:%d w:%d h:%d sw:%d sh:%d", crop_x, crop_y, crop_width, crop_height, input_video_frame->width, input_video_frame->height);
		retval = true;
	}else{
		const int diff_change = DISPQT_CONFIG_VIDEO_AUTOCROP_SENSITIVITY;
		const int change_step_x = DISPQT_CONFIG_VIDEO_AUTOCROP_STEP, change_step_y = DISPQT_CONFIG_VIDEO_AUTOCROP_STEP;
		int change_step_w, change_step_h;

		change_step_w = change_step_x * 2;
		change_step_h = change_step_y * 2;

		if((crop_x + diff_change ) < video_source_infos->sws_srccrop_x){
			video_source_infos->sws_srccrop_x -= change_step_x;  retval = true;
		}else if(crop_x > (video_source_infos->sws_srccrop_x + diff_change)){
			video_source_infos->sws_srccrop_x += change_step_x;  retval = true;
		}else{
			change_step_w = change_step_x;
		}
		if((crop_y + diff_change ) < video_source_infos->sws_srccrop_y){
			video_source_infos->sws_srccrop_y -= change_step_y;  retval = true;
		}else if(crop_y > (video_source_infos->sws_srccrop_y + diff_change)){
			video_source_infos->sws_srccrop_y += change_step_y;  retval = true;
		}else{
			change_step_h = change_step_y;
		}
		if((crop_width + diff_change ) < video_source_infos->sws_srccrop_width){
			video_source_infos->sws_srccrop_width -= change_step_w;  retval = true;
		}else if(crop_width > (video_source_infos->sws_srccrop_width + diff_change)){
			video_source_infos->sws_srccrop_width += change_step_w;  retval = true;
		}
		if((crop_height + diff_change ) < video_source_infos->sws_srccrop_height){
			video_source_infos->sws_srccrop_height -= change_step_h;  retval = true;
		}else if(crop_height > (video_source_infos->sws_srccrop_height + diff_change)){
			video_source_infos->sws_srccrop_height += change_step_h;  retval = true;
		}

		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP2 r:%d CR:%d x:%d y:%d w:%d h:%d sw:%d sw:%d cx:%d cy:%d cw:%d ch:%d sx:%d sy:%d",
				(int)retval, video_source_infos->ffmpeg_swsframe_cropreset, crop_x, crop_y, crop_width, crop_height, input_video_frame->width, input_video_frame->height,
				video_source_infos->sws_srccrop_x, video_source_infos->sws_srccrop_y, video_source_infos->sws_srccrop_width, video_source_infos->sws_srccrop_height,
				change_step_x, change_step_y);
		video_source_infos->sws_autoxcrop_framecounter = (retval)? DISPQT_CONFIG_VIDEO_AUTOCROP_SHORTDELAY : DISPQT_CONFIG_VIDEO_AUTOCROP_LONGDELAY;
	}

	return retval;

err_out_autocrop_black:
	video_source_infos->sws_autoxcrop_framecounter = DISPQT_CONFIG_VIDEO_AUTOCROP_SKIPDELAY;
	return retval;
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_crop_auto_reset()
{
	if(!video_source_infos->ffmpeg_swsframe_cropreset)
		video_source_infos->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_REFRESH;
	video_source_infos->sws_autoxcrop_framecounter = 0;
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_crop_auto_clear()
{
	video_source_infos->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_INIT;
	video_source_infos->sws_autoxcrop_framecounter = 0;
}

//-------------------------------------------------------------------------------------------------------------------------------------

static int videotrans_ffmpeg_color_range_correction(int pix_fmt, int *pixfmt_corr)
{
	int color_range = AVCOL_RANGE_JPEG;
	int pixfmt_new = pix_fmt;
	switch (pix_fmt)
	{
		case AV_PIX_FMT_YUVJ420P: pixfmt_new = AV_PIX_FMT_YUV420P; break;
		case AV_PIX_FMT_YUVJ422P: pixfmt_new = AV_PIX_FMT_YUV422P; break;
		case AV_PIX_FMT_YUVJ444P: pixfmt_new = AV_PIX_FMT_YUV444P; break;
		case AV_PIX_FMT_GRAY8:
		case AV_PIX_FMT_GRAY16LE:
		case AV_PIX_FMT_RGB555LE:
		case AV_PIX_FMT_BGR24:
		case AV_PIX_FMT_BGRA:
		case AV_PIX_FMT_BGR48LE:
		case AV_PIX_FMT_RGB48BE:
		case AV_PIX_FMT_BGRA64LE:
		case AV_PIX_FMT_RGBA64BE:
			break;
		default: color_range = AVCOL_RANGE_MPEG; break;
	}
	if(pixfmt_corr)
		*pixfmt_corr = pixfmt_new;
	return color_range;
}

int mpxplay_dispqt_videotrans_ffmpeg_swsctx_init(struct dispqt_video_filter_info_s *video_filter_infos, AVFrame *input_video_frame, struct dispqt_video_filter_format_s *output_format)
{
	int src_range, src_pix_fmt, dst_range, dst_pix_fmt, retcode;
	int output_flags = SWS_POINT;// | SWS_FULL_CHR_H_INP | SWS_ACCURATE_RND;
	const int *coefficients_input = NULL, *coefficients_output = NULL;
	enum AVColorSpace color_space_input;
	struct SwsContext *sws;

	if(!input_video_frame || (input_video_frame->width <= 0) || (input_video_frame->height <= 0) || (input_video_frame->format <= AV_PIX_FMT_NONE)
	  || (output_format->width <= 0) || (output_format->height <= 0) || ((int)output_format->pixelformat <= AV_PIX_FMT_NONE)
	){
		goto err_out_init;
	}

	//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "videowidget_ffmpeg_swsctx_init BEGIN (close)");
	mpxplay_dispqt_videotrans_ffmpeg_swsctx_close(video_filter_infos);

	src_pix_fmt = (enum AVPixelFormat)input_video_frame->format;
	src_range = (int)input_video_frame->color_range;
	color_space_input = input_video_frame->colorspace;

	if(src_range <= AVCOL_RANGE_UNSPECIFIED)
		src_range = videotrans_ffmpeg_color_range_correction(src_pix_fmt, &src_pix_fmt);

	dst_range = videotrans_ffmpeg_color_range_correction(output_format->pixelformat, &dst_pix_fmt);
	/* if context is YUV full range */
	if((dst_range < src_range) && (videotrans_ffmpeg_color_range_correction(src_pix_fmt, NULL) == videotrans_ffmpeg_color_range_correction(output_format->pixelformat, NULL)))
		dst_range = src_range;

	/* SWS_FULL_CHR_H_INT is correctly supported only for RGB formats */
	//if ((dst_pix_fmt == AV_PIX_FMT_BGR24) || (dst_pix_fmt == AV_PIX_FMT_BGRA))
	//    output_flags |= SWS_FULL_CHR_H_INT;

	sws = sws_alloc_context();
	if (!sws)
		goto err_out_init;
	video_filter_infos->sws_ctx = sws;

	av_opt_set_int(sws, "sws_flags",  output_flags, 0);

	av_opt_set_int(sws, "srcw",       input_video_frame->width,   0);
	av_opt_set_int(sws, "srch",       input_video_frame->height,  0);
	av_opt_set_int(sws, "src_format", src_pix_fmt, 0);
	av_opt_set_int(sws, "src_range",  ((src_range == AVCOL_RANGE_JPEG)? 1 : 0),   0);

	av_opt_set_int(sws, "dstw",       output_format->width,   0);
	av_opt_set_int(sws, "dsth",       output_format->height,  0);
	av_opt_set_int(sws, "dst_format", dst_pix_fmt, 0);
	av_opt_set_int(sws, "dst_range",  ((dst_range == AVCOL_RANGE_JPEG)? 1 : 0),   0);

	coefficients_input = sws_getCoefficients(color_space_input);
	coefficients_output = sws_getCoefficients(AVCOL_SPC_UNSPECIFIED);

	sws_setColorspaceDetails(sws,
							 coefficients_input, ((src_range == AVCOL_RANGE_JPEG)? 1 : 0),
							 coefficients_output, ((dst_range == AVCOL_RANGE_JPEG)? 1 : 0),
							 0, 1<<16, 1<<16);

	retcode = sws_init_context(sws, NULL, NULL);
	if(retcode < 0)
		goto err_out_init;

	video_filter_infos->ffmpeg_sws_frame = av_frame_alloc();
	if(!video_filter_infos->ffmpeg_sws_frame)
		goto err_out_init;
	video_filter_infos->ffmpeg_sws_frame->width  = output_format->width;
	video_filter_infos->ffmpeg_sws_frame->height = output_format->height;
	video_filter_infos->ffmpeg_sws_frame->format = dst_pix_fmt;

	if(av_image_alloc(&video_filter_infos->ffmpeg_sws_frame->data[0], &video_filter_infos->ffmpeg_sws_frame->linesize[0], output_format->width, output_format->height, output_format->pixelformat, DISPQT_VIDEO_PROCESS_BUFFERALIGNMENT) <= 0)
		goto err_out_init;

	video_filter_infos->input_format_cfg.width = input_video_frame->width;
	video_filter_infos->input_format_cfg.height = input_video_frame->height;
	video_filter_infos->input_format_cfg.pixelformat = (AVPixelFormat)input_video_frame->format;
	video_filter_infos->output_format_cfg.width = output_format->width;
	video_filter_infos->output_format_cfg.height = output_format->height;
	video_filter_infos->output_format_cfg.pixelformat = output_format->pixelformat;

	return MPXPLAY_ERROR_OK;

err_out_init:
	mpxplay_dispqt_videotrans_ffmpeg_swsctx_close(video_filter_infos);
	return -1;
}

void mpxplay_dispqt_videotrans_ffmpeg_swsctx_clean(struct dispqt_video_filter_info_s *video_filter_infos)
{
	video_filter_infos->sws_ctx = NULL;
	video_filter_infos->ffmpeg_sws_frame = NULL;
	video_filter_infos->ffmpeg_swsframe_hasdata = false;
	video_filter_infos->input_format_cfg.width = 0;
	video_filter_infos->input_format_cfg.height = 0;
	video_filter_infos->input_format_cfg.pixelformat = AV_PIX_FMT_NONE;
	video_filter_infos->output_format_cfg.width = 0;
	video_filter_infos->output_format_cfg.height = 0;
	video_filter_infos->output_format_cfg.pixelformat = AV_PIX_FMT_NONE;
}

void mpxplay_dispqt_videotrans_ffmpeg_swsctx_close(struct dispqt_video_filter_info_s *video_filter_infos)
{
	if(video_filter_infos->ffmpeg_sws_frame)
	{
		av_freep(&video_filter_infos->ffmpeg_sws_frame->data[0]);
		av_frame_free(&video_filter_infos->ffmpeg_sws_frame);
	}
	if(video_filter_infos->sws_ctx)
	{
		sws_freeContext(video_filter_infos->sws_ctx);
	}
	mpxplay_dispqt_videotrans_ffmpeg_swsctx_clean(video_filter_infos);
}

bool mpxplay_dispqt_videotrans_ffmpeg_swsctx_conv_frame(struct dispqt_video_filter_info_s *video_filter_infos, AVFrame *ivf, struct dispqt_video_filter_format_s *output_format)
{
	AVFrame *fsf = video_filter_infos->ffmpeg_sws_frame;

	if((output_format->width <= 0) || (output_format->height <= 0) || (output_format->pixelformat <= AV_PIX_FMT_NONE))
	{
		if(video_filter_infos->sws_ctx)
		{
			mpxplay_dispqt_videotrans_ffmpeg_swsctx_close(video_filter_infos);
		}
		return false;
	}

	if(!ivf || (ivf->width <= 0) || (ivf->height <= 0) || (ivf->format <= AV_PIX_FMT_NONE))
	{
		return false;
	}

	if(!video_filter_infos->sws_ctx || !fsf
		|| (ivf->width != video_filter_infos->input_format_cfg.width) || (ivf->height != video_filter_infos->input_format_cfg.height) || (ivf->format != video_filter_infos->input_format_cfg.pixelformat)
		|| (video_filter_infos->output_format_cfg.width != output_format->width) || (video_filter_infos->output_format_cfg.height != output_format->height) || (video_filter_infos->output_format_cfg.pixelformat != output_format->pixelformat)
	){
		if(mpxplay_dispqt_videotrans_ffmpeg_swsctx_init(video_filter_infos, ivf, output_format) != MPXPLAY_ERROR_OK)
		{
			return false;
		}
	}

	if(video_filter_infos->sws_ctx && video_filter_infos->ffmpeg_sws_frame)
	{
		if(sws_scale(video_filter_infos->sws_ctx, (uint8_t const * const *)&ivf->data[0], ivf->linesize, 0, ivf->height,
					&video_filter_infos->ffmpeg_sws_frame->data[0], &video_filter_infos->ffmpeg_sws_frame->linesize[0]) == video_filter_infos->ffmpeg_sws_frame->height
		){
			if(video_filter_infos->ffmpeg_sws_frame->data[0] && video_filter_infos->ffmpeg_sws_frame->linesize[0])
			{
				return true;
			}
		}
	}
	return false;
}

//===============================================================================================================================================================

int FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_get_sws_outformat(struct dispqt_video_filter_info_s *video_filter_infos, AVFrame *input_video_frame)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	DispQtVideoPlayer *qtvp = this->main_window->qt_video_player;
	enum AVPixelFormat sws_out_pixelformat = AV_PIX_FMT_NONE;

	if(gcfg->video_crop_type == DISPQT_VIDEOSCRCROPTYPE_AUTO)
	{
		if(input_video_frame->format != DISPQT_VIDEO_AUTOCROP_AVFORMAT)
		{
			sws_out_pixelformat = DISPQT_VIDEO_AUTOCROP_AVFORMAT;
		}
	}
	else if((qtvp->video_frame_filter_eq_process->video_filter_check(this->video_source_infos->ffmpeg_decoded_frame, AV_PIX_FMT_NONE) == AV_PIX_FMT_NONE)
	     && (qtvp->video_frame_filter_transform_process->video_filter_check(this->video_source_infos->ffmpeg_decoded_frame, AV_PIX_FMT_NONE) == AV_PIX_FMT_NONE)
	){
		if(input_video_frame->format != this->video_surface_infos->videoout_surface_pix_fmt)
		{
			sws_out_pixelformat = this->video_surface_infos->videoout_surface_pix_fmt;
		}
	}

	return sws_out_pixelformat; // returns AV_PIX_FMT_NONE if swsctx conversion is not needed
}

bool FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_conv_frame(struct dispqt_video_filter_info_s *video_filter_infos)
{
	AVFrame *ivf = (video_filter_infos->ffmpeg_transformed_frame)? video_filter_infos->ffmpeg_transformed_frame : this->video_source_infos->ffmpeg_decoded_frame;
	struct dispqt_video_filter_format_s output_format;
	output_format.width = ivf->width;
	output_format.height = ivf->height;
	output_format.pixelformat = (AVPixelFormat)videowidget_ffmpeg_swsctx_get_sws_outformat(video_filter_infos, ivf);

	return mpxplay_dispqt_videotrans_ffmpeg_swsctx_conv_frame(video_filter_infos, ivf, &output_format);
}

bool FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_check_swsframe_for_outsurface(struct dispqt_video_filter_info_s *video_filter_infos)
{   // returns true if swsctx has a valid output frame and its pixelformat matches with the video output surface (sws frame is used if all filters are disabled)
	return (video_filter_infos->ffmpeg_swsframe_hasdata && video_filter_infos->ffmpeg_sws_frame && (video_filter_infos->ffmpeg_sws_frame->format == this->video_surface_infos->videoout_surface_pix_fmt))? true : false;
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_close(void)
{
	struct dispqt_video_filter_info_s *video_filter_infos = &this->video_filter_infos;
	mpxplay_dispqt_videotrans_ffmpeg_swsctx_close(video_filter_infos);
	this->videowidget_ffmpeg_swsctx_clear(video_filter_infos);
}

void FFMpegVideoSoftwareTransform::videowidget_ffmpeg_swsctx_clear(struct dispqt_video_filter_info_s *video_filter_infos)
{
	mpxplay_dispqt_videotrans_ffmpeg_swsctx_clean(video_filter_infos);
	this->video_surface_infos->videoout_surface_clearpic = true;
	this->videowidget_ffmpeg_swsctx_crop_auto_clear();
}

#endif // MPXPLAY_LINK_ORIGINAL_FFMPEG
